@@ -12,7 +12,7 @@
  *     https://opensource.org/licenses/BSD-3-Clause
  */
 
-#define _GNU_SOURCE
+#define _GNU_SOURCE /*strndup */
 
 #include <assert.h>
 #include <ctype.h>
@@ -20,8 +20,9 @@
 #include <limits.h>
 #include <pthread.h>
 #include <string.h>
+#include <stdarg.h>
+#include <stdio.h>
 #include <stdlib.h>
-#include <sys/syscall.h>
 #include <sys/types.h>
 #include <unistd.h>
 
@@ -158,24 +159,6 @@ ly_err_clean(struct ly_ctx *ctx, struct ly_err_item *eitem)
     }
 }
 
-#ifndef  __USE_GNU
-
-char *
-get_current_dir_name(void)
-{
-    char tmp[PATH_MAX];
-    char *retval;
-
-    if (getcwd(tmp, sizeof(tmp))) {
-        retval = strdup(tmp);
-        LY_CHECK_ERR_RETURN(!retval, LOGMEM(NULL), NULL);
-        return retval;
-    }
-    return NULL;
-}
-
-#endif
-
 const char *
 strpbrk_backwards(const char *s, const char *accept, unsigned int s_len)
 {
