@@ -20,26 +20,34 @@ from _libyang import ffi
 from _libyang import lib
 
 from .data import DNode
-from .data import PathOpt
+from .data import data_format
+from .data import parser_flags
+from .data import path_flags
 from .schema import Module
 from .schema import SNode
+from .schema import schema_in_format
 from .util import LibyangError
 from .util import c2str
 from .util import str2c
 
 
+LOG = logging.getLogger(__name__)
+LOG.addHandler(logging.NullHandler())
+
+
 #------------------------------------------------------------------------------
 class Context(object):
 
-    def __init__(self, search_path=None,
-                 options=lib.LY_CTX_DISABLE_SEARCHDIR_CWD,
-                 pointer=None):
+    def __init__(self, search_path=None, disable_searchdir_cwd=True, pointer=None):
         if pointer is not None:
             self._ctx = ffi.cast('struct ly_ctx *', pointer)
             return  # already initialized
 
-        self._ctx = ffi.gc(lib.ly_ctx_new(ffi.NULL, options),
-                           lambda c: lib.ly_ctx_destroy(c, ffi.NULL))
+        options = 0
+        if disable_searchdir_cwd:
+            options |= lib.LY_CTX_DISABLE_SEARCHDIR_CWD
+
+        self._ctx = lib.ly_ctx_new(ffi.NULL, options)
         if not self._ctx:
             raise self.error('cannot create context')
 
@@ -59,31 +67,54 @@ class Context(object):
             if lib.ly_ctx_set_searchdir(self._ctx, str2c(path)) != 0:
                 raise self.error('cannot set search dir')
 
-    def error(self, msg, *args):
-        errors = []
-        try:
-            err = lib.ly_err_first(self._ctx)
-            while err:
-                e = []
-                if err.path:
-                    e.append(c2str(err.path))
-                if err.msg:
-                    e.append(c2str(err.msg))
-                if err.apptag:
-                    e.append(c2str(err.apptag))
-                if e:
-                    errors.append(': '.join(e))
-                err = err.next
-        finally:
-            lib.ly_err_clean(self._ctx, ffi.NULL)
+    def destroy(self):
+        if self._ctx is not None:
+            lib.ly_ctx_destroy(self._ctx, ffi.NULL)
+            self._ctx = None
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args, **kwargs):
+        self.destroy()
+
+    def error(self, msg, *args):
         msg %= args
-        if errors:
-            msg += ': ' + ' '.join(errors)
+
+        if self._ctx:
+            err = lib.ly_err_first(self._ctx)
+            if err:
+                if err.msg:
+                    msg += ': %s' % c2str(err.msg)
+                if err.path:
+                    msg += ': %s' % c2str(err.path)
+            lib.ly_err_clean(self._ctx, ffi.NULL)
 
         return LibyangError(msg)
 
+    def parse_module_file(self, fileobj, fmt='yang'):
+        if self._ctx is None:
+            raise RuntimeError('context already destroyed')
+        fmt = schema_in_format(fmt)
+        mod = lib.lys_parse_fd(self._ctx, fileobj.fileno(), fmt)
+        if not mod:
+            raise self.error('cannot parse module')
+
+        return Module(self, mod)
+
+    def parse_module_str(self, s, fmt='yang'):
+        if self._ctx is None:
+            raise RuntimeError('context already destroyed')
+        fmt = schema_in_format(fmt)
+        mod = lib.lys_parse_mem(self._ctx, str2c(s), fmt)
+        if not mod:
+            raise self.error('cannot parse module')
+
+        return Module(self, mod)
+
     def load_module(self, name):
+        if self._ctx is None:
+            raise RuntimeError('context already destroyed')
         mod = lib.ly_ctx_load_module(self._ctx, str2c(name), ffi.NULL)
         if not mod:
             raise self.error('cannot load module')
@@ -91,6 +122,8 @@ class Context(object):
         return Module(self, mod)
 
     def get_module(self, name):
+        if self._ctx is None:
+            raise RuntimeError('context already destroyed')
         mod = lib.ly_ctx_get_module(self._ctx, str2c(name), ffi.NULL, False)
         if not mod:
             raise self.error('cannot get module')
@@ -98,6 +131,8 @@ class Context(object):
         return Module(self, mod)
 
     def find_path(self, path):
+        if self._ctx is None:
+            raise RuntimeError('context already destroyed')
         node_set = lib.ly_ctx_find_path(self._ctx, str2c(path))
         if not node_set:
             raise self.error('cannot find path')
@@ -107,16 +142,22 @@ class Context(object):
         finally:
             lib.ly_set_free(node_set)
 
-    def create_data_path(self, path, parent=None, value=None, flags=0):
+    def create_data_path(self, path, parent=None, value=None, rpc_output=False):
+        if self._ctx is None:
+            raise RuntimeError('context already destroyed')
         lib.lypy_set_errno(0)
-        if value is not None and not isinstance(value, str):
-            value = str(value)
+        if value is not None:
+            if isinstance(value, bool):
+                value = str(value).lower()
+            elif not isinstance(value, str):
+                value = str(value)
+        flags = path_flags(
+            update=True, no_parent_ret=True, rpc_output=rpc_output)
         dnode = lib.lyd_new_path(
             parent._node if parent else ffi.NULL,
-            self._ctx, str2c(path), str2c(value), 0,
-            PathOpt.UPDATE | PathOpt.NOPARENTRET | flags)
+            self._ctx, str2c(path), str2c(value), 0, flags)
         if lib.lypy_get_errno() != 0:
-            raise self.error('cannot create data path: %s', path)
+            raise self.error('cannot create data path')
 
         if not dnode and parent:
             # This can happen when path points to an already created leaf and
@@ -135,13 +176,27 @@ class Context(object):
 
         return DNode.new(self, dnode)
 
-    def parse_data_str(self, s, fmt, flags=0):
+    def parse_data_str(self, s, fmt, data=False, config=False, strict=False,
+                       trusted=False, no_yanglib=False):
+        if self._ctx is None:
+            raise RuntimeError('context already destroyed')
+        flags = parser_flags(
+            data=data, config=config, strict=strict, trusted=trusted,
+            no_yanglib=no_yanglib)
+        fmt = data_format(fmt)
         dnode = lib.lyd_parse_mem(self._ctx, str2c(s), fmt, flags)
         if not dnode:
             raise self.error('failed to parse data tree')
         return DNode.new(self, dnode)
 
-    def parse_data_file(self, fileobj, fmt, flags=0):
+    def parse_data_file(self, fileobj, fmt, data=False, config=False,
+                        strict=False, trusted=False, no_yanglib=False):
+        if self._ctx is None:
+            raise RuntimeError('context already destroyed')
+        flags = parser_flags(
+            data=data, config=config, strict=strict, trusted=trusted,
+            no_yanglib=no_yanglib)
+        fmt = data_format(fmt)
         dnode = lib.lyd_parse_fd(self._ctx, fileobj.fileno(), fmt, flags)
         if not dnode:
             raise self.error('failed to parse data tree')
@@ -151,6 +206,8 @@ class Context(object):
         """
         Return an iterator that yields all implemented modules from the context
         """
+        if self._ctx is None:
+            raise RuntimeError('context already destroyed')
         idx = ffi.new('uint32_t *')
         mod = lib.ly_ctx_get_module_iter(self._ctx, idx)
         while mod:
@@ -178,18 +235,36 @@ def libyang_c_logging_callback(level, msg, path):
     LOG.log(LOG_LEVELS.get(level, logging.NOTSET), fmt, *args)
 
 
-def set_log_level(level):
+def configure_logging(enable_py_logger, level=logging.ERROR):
+    """
+    Configure libyang logging behaviour.
+
+    :arg bool enable_py_logger:
+        If False, configure libyang to store the errors in the context until
+        they are consumed when Context.error() is called. This is the default
+        behaviour.
+
+        If True, libyang log messages will be sent to the python 'libyang'
+        logger and will be processed according to the python logging
+        configuration. Note that by default, the 'libyang' python logger is
+        created with a NullHandler() which means that all messages are lost
+        until another handler is configured for that logger.
+    :arg int level:
+        Python logging level. By default only ERROR messages are stored/logged.
+    """
     for ly_lvl, py_lvl in LOG_LEVELS.items():
         if py_lvl == level:
             lib.ly_verb(ly_lvl)
-            return
+            break
+    if enable_py_logger:
+        lib.ly_log_options(lib.LY_LOLOG |lib.LY_LOSTORE)
+        lib.ly_set_log_clb(lib.lypy_log_cb, True)
+    else:
+        lib.ly_log_options(lib.LY_LOSTORE)
+        lib.ly_set_log_clb(ffi.NULL, False)
 
 
-set_log_level(logging.ERROR)
-lib.ly_set_log_clb(lib.lypy_log_cb, True)
-lib.ly_log_options(lib.LY_LOLOG | lib.LY_LOSTORE)
-LOG = logging.getLogger(__name__)
-LOG.addHandler(logging.NullHandler())
+configure_logging(False, logging.ERROR)
 
 
 #------------------------------------------------------------------------------
