@@ -8,6 +8,30 @@ from .util import c2str
 from .util import str2c
 
 
+#------------------------------------------------------------------------------
+def schema_in_format(fmt_string):
+    if fmt_string == 'yang':
+        return lib.LYS_IN_YANG
+    if fmt_string == 'yin':
+        return lib.LYS_IN_YIN
+    raise ValueError('unknown schema input format: %r' % fmt_string)
+
+
+#------------------------------------------------------------------------------
+def schema_out_format(fmt_string):
+    if fmt_string == 'yang':
+        return lib.LYS_OUT_YANG
+    if fmt_string == 'yin':
+        return lib.LYS_OUT_YIN
+    if fmt_string == 'tree':
+        return lib.LYS_OUT_TREE
+    if fmt_string == 'info':
+        return lib.LYS_OUT_INFO
+    if fmt_string == 'json':
+        return lib.LYS_OUT_JSON
+    raise ValueError('unknown schema output format: %r' % fmt_string)
+
+
 #------------------------------------------------------------------------------
 class Module(object):
 
@@ -27,6 +51,9 @@ class Module(object):
     def filepath(self):
         return c2str(self._module.filepath)
 
+    def implemented(self):
+        return bool(lib.lypy_module_implemented(self._module))
+
     def feature_enable(self, name):
         ret = lib.lys_features_enable(self._module, str2c(name))
         if ret != 0:
@@ -72,14 +99,19 @@ class Module(object):
     def __str__(self):
         return self.dump_str()
 
-    def dump_str(self, fmt=lib.LYS_OUT_TREE, path=None):
+    def dump_str(self, fmt='tree', path=None):
+        fmt = schema_out_format(fmt)
         buf = ffi.new('char **')
         ret = lib.lys_print_mem(buf, self._module, fmt, str2c(path), 0, 0)
         if ret != 0:
             raise self.context.error('cannot print module')
-        return c2str(buf[0])
+        try:
+            return c2str(buf[0])
+        finally:
+            lib.free(buf[0])
 
-    def dump_file(self, fileobj, fmt=lib.LYS_OUT_TREE, path=None):
+    def dump_file(self, fileobj, fmt='tree', path=None):
+        fmt = schema_out_format(fmt)
         ret = lib.lys_print_fd(
             fileobj.fileno(), self._module, fmt, str2c(path), 0, 0)
         if ret != 0:
@@ -562,7 +594,7 @@ class IfOrFeatures(IfFeatureExprTree):
 
 
 #------------------------------------------------------------------------------
-class Node(object):
+class SNode(object):
 
     CONTAINER = lib.LYS_CONTAINER
     LEAF = lib.LYS_LEAF
@@ -595,7 +627,11 @@ class Node(object):
         return c2str(self._node.name)
 
     def fullname(self):
-        return c2str(ffi.gc(lib.lypy_node_fullname(self._node), lib.free))
+        try:
+            s = lib.lypy_node_fullname(self._node)
+            return c2str(s)
+        finally:
+            lib.free(s)
 
     def description(self):
         return c2str(self._node.dsc)
@@ -629,10 +665,18 @@ class Node(object):
         return Module(self.context, module_p)
 
     def schema_path(self):
-        return c2str(ffi.gc(lib.lys_path(self._node, 0), lib.free))
+        try:
+            s = lib.lys_path(self._node, 0)
+            return c2str(s)
+        finally:
+            lib.free(s)
 
     def data_path(self):
-        return c2str(ffi.gc(lib.lypy_data_path_pattern(self._node), lib.free))
+        try:
+            s = lib.lypy_data_path_pattern(self._node)
+            return c2str(s)
+        finally:
+            lib.free(s)
 
     def extensions(self):
         for i in range(self._node.ext_size):
@@ -652,10 +696,10 @@ class Node(object):
 
     def parent(self):
         parent_p = lib.lys_parent(self._node)
-        while parent_p and parent_p.nodetype not in Node.NODETYPE_CLASS:
+        while parent_p and parent_p.nodetype not in SNode.NODETYPE_CLASS:
             parent_p = lib.lys_parent(parent_p)
         if parent_p:
-            return Node.new(self.context, parent_p)
+            return SNode.new(self.context, parent_p)
         return None
 
     def __repr__(self):
@@ -676,16 +720,16 @@ class Node(object):
 
     @classmethod
     def new(cls, context, node_p):
-        nodecls = cls.NODETYPE_CLASS.get(node_p.nodetype, Node)
+        nodecls = cls.NODETYPE_CLASS.get(node_p.nodetype, SNode)
         return nodecls(context, node_p)
 
 
 #------------------------------------------------------------------------------
-@Node.register(Node.LEAF)
-class Leaf(Node):
+@SNode.register(SNode.LEAF)
+class SLeaf(SNode):
 
     def __init__(self, context, node_p):
-        Node.__init__(self, context, node_p)
+        SNode.__init__(self, context, node_p)
         self._leaf = ffi.cast('struct lys_node_leaf *', node_p)
 
     def default(self):
@@ -711,11 +755,11 @@ class Leaf(Node):
 
 
 #------------------------------------------------------------------------------
-@Node.register(Node.LEAFLIST)
-class LeafList(Node):
+@SNode.register(SNode.LEAFLIST)
+class SLeafList(SNode):
 
     def __init__(self, context, node_p):
-        Node.__init__(self, context, node_p)
+        SNode.__init__(self, context, node_p)
         self._leaflist = ffi.cast('struct lys_node_leaflist *', node_p)
 
     def ordered(self):
@@ -740,11 +784,11 @@ class LeafList(Node):
 
 
 #------------------------------------------------------------------------------
-@Node.register(Node.CONTAINER)
-class Container(Node):
+@SNode.register(SNode.CONTAINER)
+class SContainer(SNode):
 
     def __init__(self, context, node_p):
-        Node.__init__(self, context, node_p)
+        SNode.__init__(self, context, node_p)
         self._container = ffi.cast('struct lys_node_container *', node_p)
 
     def presence(self):
@@ -762,11 +806,11 @@ class Container(Node):
 
 
 #------------------------------------------------------------------------------
-@Node.register(Node.LIST)
-class List(Node):
+@SNode.register(SNode.LIST)
+class SList(SNode):
 
     def __init__(self, context, node_p):
-        Node.__init__(self, context, node_p)
+        SNode.__init__(self, context, node_p)
         self._list = ffi.cast('struct lys_node_list *', node_p)
 
     def ordered(self):
@@ -782,7 +826,7 @@ class List(Node):
     def keys(self):
         for i in range(self._list.keys_size):
             node = ffi.cast('struct lys_node *', self._list.keys[i])
-            yield Leaf(self.context, node)
+            yield SLeaf(self.context, node)
 
     def must_conditions(self):
         for i in range(self._list.must_size):
@@ -794,9 +838,9 @@ class List(Node):
 
 
 #------------------------------------------------------------------------------
-@Node.register(Node.INPUT)
-@Node.register(Node.OUTPUT)
-class RpcInOut(Node):
+@SNode.register(SNode.INPUT)
+@SNode.register(SNode.OUTPUT)
+class SRpcInOut(SNode):
 
     def __iter__(self):
         return self.children()
@@ -809,8 +853,8 @@ class RpcInOut(Node):
 
 
 #------------------------------------------------------------------------------
-@Node.register(Node.RPC)
-class Rpc(Node):
+@SNode.register(SNode.RPC)
+class SRpc(SNode):
 
     def must_conditions(self):
         return ()
@@ -865,5 +909,16 @@ def iter_children(context, parent, skip_keys=False, types=None, options=0):
     child = lib.lys_getnext(ffi.NULL, parent, module, options)
     while child:
         if not _skip(child):
-            yield Node.new(context, child)
+            yield SNode.new(context, child)
         child = lib.lys_getnext(child, parent, module, options)
+
+
+#------------------------------------------------------------------------------
+# compat
+Container = SContainer
+Leaf = SLeaf
+LeafList = SLeafList
+List = SList
+Node = SNode
+Rpc = SRpc
+RpcInOut = SRpcInOut
